/**
 *  @file MyPoincareMap.h
 *
 *  @author kapela  @date 2010-11-23
 */

#ifndef MYPOINCAREMAP_H_
#define MYPOINCAREMAP_H_
#include "capd/mpcapdlib.h"
#include "capd/poincare/PoincareMap.h"

template <typename DynSysT>
class MyPoincareMap : public capd::poincare::PoincareMap<DynSysT> {
public:
  typedef DynSysT DynamicalSystemType;
  typedef capd::poincare::PoincareMap<DynSysT> BaseClass;
  typedef typename DynamicalSystemType::MapType MapType;
  typedef typename MapType::FunctionType FunctionType;
  typedef typename MapType::MatrixType MatrixType;
  typedef typename MapType::VectorType VectorType;
  typedef typename MapType::ScalarType ScalarType;
  typedef typename ScalarType::BoundType BoundType;

  MyPoincareMap( DynamicalSystemType  & ds,
                 FunctionType & section,
                 capd::poincare::CrossingDirection direction = capd::poincare::Both,
                 const BoundType & errorTolerance = 0.1
  ) :  BaseClass(ds, section, direction, errorTolerance){
  }
  VectorType operator()(const VectorType & x, int n = 1){
    capd::C0Rect2Set set(x);
    return BaseClass::operator ()(set, n);
  }
  VectorType operator()(const VectorType & x, MatrixType & der, int n = 1){
    capd::vectalg::EuclLNorm<VectorType, MatrixType> norm;
      capd::C1Rect2Set set(x, norm);
      return BaseClass::operator ()(set, der, n);
  }
private:
};


#endif /* MYPOINCAREMAP_H_ */
