/**
 *  @file improveIC.h
 *
 *  @author kapela  @date 2011-01-11
 */

#ifndef IMPROVEIC_H_
#define IMPROVEIC_H_

#include "capd/newton/Krawczyk.hpp"
#include "ParallelMap.h"
#include "Parameters.h"


/**
 *  refines initial conditions until error is less that maxError or maxN iterates is reached
 */
template <typename PoincareMap>
typename PoincareMap::VectorType refineICParallel(
    PoincareMap & pm,                             ///< Poincare map
    int nIterates,                                ///< number of iterates of Poincare map
    const typename PoincareMap::VectorType &  x0, ///< approximated initial condition on the first section
    int maxN,                                     ///< maximal number of iterates of refining process
    typename PoincareMap::ScalarType  maxError    ///< requested error tolerance
){
  typedef typename PoincareMap::VectorType VectorType;
  ParallelMap<PoincareMap> dmap(pm, nIterates);
  VectorType x1 = dmap.findIC(x0);
  x1 = dmap.refineIC(x1, maxN, maxError);
  return x1;
}



/// improves enclosures for initial condition by krawczyk operator iteration
template <typename PoincareMap>
void improveRigoorousIC(
    PoincareMap & map,                             ///< Poincare map
    int nIterates,                                ///< number of iterates of Poincare map
    typename PoincareMap::VectorType &  X,        ///< initial condition
    int numberOfImprovements                      ///< number of iterations of Krawczyk operator computations to improve estimates of i.c.
){
  typedef typename PoincareMap::VectorType VectorType;


  std::cout << "\n X " << X << "\n N " << numberOfImprovements << std::endl;
  VectorType x0 = capd::vectalg::midVector(X);

  capd::newton::Krawczyk< PoincareMap> krawczyk(map);

  for(int i=0; i<numberOfImprovements; i++ ){
    //    IVector  K = krawczyk.KrawczykOperator(capd::vectalg::midVector(krawczyk.K), krawczyk.K);
    std::cout << " computation of krawczyk operator \n   x0 : " << x0 << "\n   X : " << X << std::endl;
    VectorType  K = krawczyk.KrawczykOperator(x0, X);
    std::cout
    << "\n X = "<< X << "\n diam X " << diam(X)
    << "\n K = "<< K << "\n diam K " << diam(K) << std::endl;
    if(! capd::vectalg::intersection(K, X, X)){
      std::cout << "\n\n INTERSECTION IS EMPTY in improveRigorousIC ";
      exit(1);
    }
  }
}



#endif /* IMPROVEIC_H_ */
