#include <string>
#include <sstream>

std::string onePart3D(int index,int numberOfBodies)
{
   std::ostringstream result;
   int i;
   for(i=1;i<=numberOfBodies;i++)
   {
      if(i==index) continue;
      result << "(q" << i << 1 << "-q" << index <<"1)";
      result << "/sqr("
             << "(q" << i << "1-q" << index << "1)^2+"
             << "(q" << i << "2-q" << index << "2)^2+"
             << "(q" << i << "3-q" << index << "3)^2"
             << ")^3";
      if(! (i==numberOfBodies || (i==numberOfBodies-1 && index==numberOfBodies)))
         result << "+";
   }
   result << ",";
   for(i=1;i<=numberOfBodies;i++)
   {
      if(i==index) continue;
      result << "(q" << i << 2 << "-q" << index <<"2)";
      result << "/sqrt("
             << "(q" << i << "1-q" << index << "1)^2+"
             << "(q" << i << "2-q" << index << "2)^2+"
             << "(q" << i << "3-q" << index << "3)^2"
             << ")^3";
      if(! (i==numberOfBodies || (i==numberOfBodies-1 && index==numberOfBodies)))
         result << "+";
   }
   result << ",";
   for(i=1;i<=numberOfBodies;i++)
   {
      if(i==index) continue;
      result << "(q" << i << 3 << "-q" << index <<"3)";
      result << "/sqrt("
             << "(q" << i << "1-q" << index << "1)^2+"
             << "(q" << i << "2-q" << index << "2)^2+"
             << "(q" << i << "3-q" << index << "3)^2"
             << ")^3";
      if(! (i==numberOfBodies || (i==numberOfBodies-1 && index==numberOfBodies)))
         result << "+";
   }
   return result.str();
}

std::string nBodyEquation3D(int numberOfBodies)
{
   std::ostringstream var;
   var << "var:";
   int i;
   for(i=1;i<=numberOfBodies;i++)
   {
      var << "q" << i << "1" << ","
          << "q" << i << "2" << ","
          << "q" << i << "3" << ",";
   }
   for(i=1;i<=numberOfBodies;i++)
   {
      var << "p" << i << "1" << ","
          << "p" << i << "2" << ","
          << "p" << i << "3";
      if(i!=numberOfBodies)
         var << ",";
      else
         var << ";";
   }

   var << "fun:";
   for(i=1;i<=numberOfBodies;i++)
   {
      var << "p" << i << "1" << ","
          << "p" << i << "2" << ","
          << "p" << i << "3" << ",";
   }

   for(i=1;i<=numberOfBodies;i++)
   {
      var << onePart3D(i,numberOfBodies);
      if(i!=numberOfBodies)
         var << ",";
      else
         var << ";";
   }
   return var.str();
}


std::string oneParticle2D(int index,int numberOfBodies)
{
   std::ostringstream result;
   int i;
   for(i=1;i<=numberOfBodies;i++)
   {
      if(i==index) continue;
      result << "(q" << i << 1 << "-q" << index <<"1)";
      result << "/(sqrt("
             << "(q" << i << "1-q" << index << "1)^2+"
             << "(q" << i << "2-q" << index << "2)^2"
             << "))^3";
      if(! (i==numberOfBodies || (i==numberOfBodies-1 && index==numberOfBodies)))
         result << "+";
   }
   result << ",";
   for(i=1;i<=numberOfBodies;i++)
   {
      if(i==index) continue;
      result << "(q" << i << 2 << "-q" << index <<"2)";
      result << "/(sqrt("
             << "(q" << i << "1-q" << index << "1)^2+"
             << "(q" << i << "2-q" << index << "2)^2"
             << "))^3";
      if(! (i==numberOfBodies || (i==numberOfBodies-1 && index==numberOfBodies)))
         result << "+";
   }
   return result.str();
}

std::string nBodyEquation2D(int numberOfBodies)
{
   std::ostringstream var;
   var << "var:";
   int i;
   for(i=1;i<=numberOfBodies;i++)
   {
      var << "q" << i << "1" << ","
          << "q" << i << "2" << ","
          << "p" << i << "1" << ","
          << "p" << i << "2";
      if(i!=numberOfBodies)
         var << ",";
      else
         var << ";";
   }

   var << "fun:";

   for(i=1;i<=numberOfBodies;i++)
   {
      var << "p" << i << "1" << ","
          << "p" << i << "2" << ",";
      var << oneParticle2D(i,numberOfBodies);
      if(i!=numberOfBodies)
         var << ",";
      else
         var << ";";
   }
   return var.str();
}


/*************************************************************************/
//
//    Planar N-body Equation reduced by center of the mass condition
//
//    variables are
//      (q11,q12,p11,p12,q21,q22,...,q(N-1)1,q(N-1)2,p(N-1)1,p(N-1)2)
//
//    q - positions,  p - velocities
//
/*************************************************************************/


// It gives  'coord' coordinate of the last particle computed from the center of mass condition
// if coord is greater than 2 then we put velocities coordinates.

std::string lastParticle(int coord, int numberOfBodies)
{
   std::ostringstream result;
   int i;
   char leter = 'q';
   if(coord>=3)
   {
       coord-=2;
       leter = 'p';
   }

   result <<"(";
   for(i=1; i<=numberOfBodies-1; ++i)
   {
     result << "-" << leter << i << coord;
   }
   result << ")";

   return result.str();
}

#define __NEW_NBODY__
#ifdef __NEW_NBODY__
/*
std::string reducedOneParticle2D(int index,int numberOfBodies)
{
   std::ostringstream result;

   std::string qN[2];
   qN[0] = lastParticle(1, numberOfBodies);
   qN[1] = lastParticle(2, numberOfBodies);

   int i, j;
   for(j=1; j<=2; ++j)
   {
      for(i=1;i<=numberOfBodies-1; ++i)
      {
         if(i==index) continue;
         result << "(q" << i << j <<"-q" << index << j <<")";
         result << "*(sqrt("
                << "(q" << i << "1-q" << index << "1)*(q" << i << "1-q" << index << "1)+"
                << "(q" << i << "2-q" << index << "2)*(q" << i << "2-q" << index << "2)"
                << ")^(-3))";
         result << "+";
      }
      // Last body
      result << "(" << qN[j-1] <<"-q"<< index << j;
      result << ")*(sqrt(";
      result << "(q" << index << "1-" << qN[0] << ")*(q" << index << "1-" << qN[0] << ")+"
             << "(q" << index << "2-" << qN[1] << ")*(q" << index << "2-" << qN[1] << ")"
             << ")^(-3))";

      if (j==1) result << ",";
   }
   return result.str();
}
*/
std::string reducedOneParticle2D(int index,int numberOfBodies)
{
   std::ostringstream result;

   std::string qN[2];
   qN[0] = lastParticle(1, numberOfBodies);
   qN[1] = lastParticle(2, numberOfBodies);

   int i, j;
   for(j=1; j<=2; ++j)
   {
      for(i=1;i<=numberOfBodies-1; ++i)
      {
         if(i==index) continue;
         result << "(q" << i << j <<"-q" << index << j <<")";
         result << "*(("
                << "(q" << i << "1-q" << index << "1)^2+"
                << "(q" << i << "2-q" << index << "2)^2"
                << ")^(-1.5))";
         result << "+";
      }
      // Last body
      result << "(" << qN[j-1] <<"-q"<< index << j;
      result << ")*((";
      result << "(q" << index << "1-" << qN[0] << ")^2+"
             << "(q" << index << "2-" << qN[1] << ")^2"
             << ")^(-1.5))";
      if (j==1) result << ",";
   }

   return result.str();
}
#else
std::string reducedOneParticle2D(int index,int numberOfBodies)
{
   std::ostringstream result;

   std::string qN[2];
   qN[0] = lastParticle(1, numberOfBodies);
   qN[1] = lastParticle(2, numberOfBodies);

   int i, j;
   for(j=1; j<=2; ++j)
   {
      for(i=1;i<=numberOfBodies-1; ++i)
      {
         if(i==index) continue;
         result << "(q" << i << j <<"-q" << index << j <<")";
         result << "*(("
                << "(q" << i << "1-q" << index << "1)*(q" << i << "1-q" << index << "1)+"
                << "(q" << i << "2-q" << index << "2)*(q" << i << "2-q" << index << "2)"
                << ")^(-1.5))";
         result << "+";
      }
      // Last body
      result << "(" << qN[j-1] <<"-q"<< index << j;
      result << ")*((";
      result << "(q" << index << "1-" << qN[0] << ")*(q" << index << "1-" << qN[0] << ")+"
             << "(q" << index << "2-" << qN[1] << ")*(q" << index << "2-" << qN[1] << ")"
             << ")^(-1.5))";

      if (j==1) result << ",";
   }

   return result.str();
}
#endif

/*********************************************************************
  Generates 2D N-body equation reduced by center of mass condition
*********************************************************************/

std::string reducedNBodyEquation2D(int numberOfBodies)
{
   std::ostringstream var;
   var << "var:";
   int i;
   for(i=1;i<=numberOfBodies-1;i++)
   {
      var << "q" << i << "1" << ","
          << "q" << i << "2" << ",";
      var << "p" << i << "1" << ","
          << "p" << i << "2";
      if(i!=numberOfBodies-1)
         var << ",";
      else
         var << ";";
   }

   var << "fun:";
   for(i=1;i<=numberOfBodies-1;i++)
   {
      var << "p" << i << "1" << ","
          << "p" << i << "2" << ",";
      var << reducedOneParticle2D(i,numberOfBodies);

      if(i!=numberOfBodies-1)
         var << ",";
      else
         var << ";";
   }
   return var.str();
}

/*
std::string reduced3BodyEquation2D(){
  std::ostringstream str;
  str << "var:q11,q12,p11,p12,q21,q22,p21,p22;"
      << "fun:2*p11+p21,2*p12+p22,"
      << "(q21-q11)*(((q21-q11)^2+(q22-q12)^2)^(-1.5))-2*(2*q11+q21)*((((2*q11+q21)^2+(2*q12+q22))^2)^(-1.5))-(q11+2*q21)*((((q11+2*q21)^2+(q12+2*q22))^2)^(-1.5)),"
      << "(q22-q12)*(((q21-q11)^2+(q22-q12)^2)^(-1.5))-2*(2*q12+q22)*((((2*q11+q21)^2+(2*q12+q22))^2)^(-1.5))-(q12+2*q22)*((((q11+2*q21)^2+(q12+2*q22))^2)^(-1.5)),"
      << "2*p21+p11,2*p22+p12,"
      << "(q11-q21)*(((q21-q11)^2+(q22-q12)^2)^(-1.5))-(2*q11+q21)*((((2*q11+q21)^2+(2*q12+q22))^2)^(-1.5))-2*(q11+2*q21)*((((q11+2*q21)^2+(q12+2*q22))^2)^(-1.5)),"
      << "(q12-q22)*(((q21-q11)^2+(q22-q12)^2)^(-1.5))-(2*q12+q22)*((((2*q11+q21)^2+(2*q12+q22))^2)^(-1.5))-2*(q12+2*q22)*((((q11+2*q21)^2+(q12+2*q22))^2)^(-1.5));";
  return str.str();
}
*/
std::string reduced3BodyEquation2D(){
  std::ostringstream str;
  str << "var:x1,y1,px1,py1,x2,y2,px2,py2;fun:"
      << "2*px1+px2,"
      << "2*py1+py2,"
      << "(x2-x1)*(((x1-x2)^2+(y1-y2)^2)^(-1.5))-2*(2*x1+x2)*(((2*x1+x2)^2+(2*y1+y2)^2)^(-1.5))-(x1+2*x2)*(((x1+2*x2)^2+(y1+2*y2)^2)^(-1.5)),"
      << "(y2-y1)*(((x1-x2)^2+(y1-y2)^2)^(-1.5))-2*(2*y1+y2)*(((2*x1+x2)^2+(2*y1+y2)^2)^(-1.5))-(y1+2*y2)*(((x1+2*x2)^2+(y1+2*y2)^2)^(-1.5)),"
      << "px1+2*px2,"
      << "py1+2*py2,"
      << "(x1-x2)*(((x1-x2)^2+(y1-y2)^2)^(-1.5))-(2*x1+x2)*(((2*x1+x2)^2+(2*y1+y2)^2)^(-1.5))-2*(x1+2*x2)*(((x1+2*x2)^2+(y1+2*y2)^2)^(-1.5)),"
      << "(y1-y2)*(((x1-x2)^2+(y1-y2)^2)^(-1.5))-(2*y1+y2)*(((2*x1+x2)^2+(2*y1+y2)^2)^(-1.5))-2*(y1+2*y2)*(((x1+2*x2)^2+(y1+2*y2)^2)^(-1.5));";
  return str.str();
}

//******************************************************************************/
// Generates First Integrals Function (angular momentum, total energy)
//   for system after 'center of the mass' reduction
//******************************************************************************/

std::string reducedKineticEnergy(int numberOfBodies)
{
   std::ostringstream var;
   std::string    pN1 = lastParticle(3, numberOfBodies),
                  pN2 = lastParticle(4, numberOfBodies);

   for(int i=1; i<=numberOfBodies-1; i++)
   {
      var << "("
          << "p" << i <<"1*p" << i<<"1+"
          << "p" << i <<"2*p" << i<<"2"
          << ")/2+";
   }

   // Last body
   var << "("
       << pN1 <<"*" << pN1 << "+"
       << pN2 <<"*" << pN2
       << ")/2";

   return var.str();
}


std::string reducedPotentialEnergy(int numberOfBodies)
{
   std::ostringstream var;

   std::string   qN1 = lastParticle(1, numberOfBodies),
                 qN2 = lastParticle(2, numberOfBodies);

   var << "(";
   for(int i=1; i<=numberOfBodies-1; ++i)
   {
      for(int j=i+1; j<numberOfBodies; ++j)
      {
         var << "(("
             << "(q" << j << "1-q" << i << "1)*(q" << j << "1-q" << i << "1)+"
             << "(q" << j << "2-q" << i << "2)*(q" << j << "2-q" << i << "2)"
             << ")^(-0.5))+";
      }
      // Last body
      var << "(("
          << "(" << qN1 << "-q" << i << "1)*(" << qN1 << "-q" << i << "1)+"
          << "(" << qN2 << "-q" << i << "2)*(" << qN2 << "-q" << i << "2)"
          << ")^(-0.5))";

      if (i!=numberOfBodies-1)
         var << "+";
   }
   var <<")";

   return var.str();
}


std::string reducedAngularMomentum(int numberOfBodies)
{
   std::ostringstream var;

   std::string     qN1 = lastParticle(1, numberOfBodies),
                   qN2 = lastParticle(2, numberOfBodies),
                   pN1 = lastParticle(3, numberOfBodies),
                   pN2 = lastParticle(4, numberOfBodies);

    // Angular momentum - planar case

   var << "(";
   for(int i=1; i<numberOfBodies; i++)
   {
      var << "q" << i << "1*p" << i << "2-"
          << "q" << i << "2*p" << i << "1+";
   }
   var << qN1 << "*" << pN2 << "-"
       << qN2 << "*" << pN1 << ")";

   return var.str();

}

std::string reducedFirstIntegrals2D(int numberOfBodies)
{

   std::ostringstream var;
   var << "var:";
   int i;
   for(i=1;i<=numberOfBodies-1;i++)
   {
      var << "q" << i << "1" << ","
          << "q" << i << "2" << ",";
      var << "p" << i << "1" << ","
          << "p" << i << "2";
      if(i!=numberOfBodies-1)
         var << ",";
      else
         var << ";";
   }

   var << "fun:" << reducedKineticEnergy(numberOfBodies)
       << "-" << reducedPotentialEnergy(numberOfBodies) << ","
       << reducedAngularMomentum(numberOfBodies);

   for(i=0; i<((4*numberOfBodies)-6); ++i)
      var << ",0";
   var << ";" << std::ends;

   return var.str();
}

std::string whittaker3BodyEquation2D(){
   return "var:q1,q2,q3,p1,p2,p3;par:M;"
          "fun:2*p1+p2,"
          "p1+2*p2+(p3*q3)/q1-(2*q3*(p3*q2-p2*q3-M))*q1^(-2),"
          "2*p3-(p3*q2)/q1-(p3*q2-p2*q3-M)/q1+(2*q2*(p3*q2-p2*q3-M))*q1^(-2),"
          "-(1+p3*(p3*q2-p2*q3-M))*q1^(-2)+(2*(p3*q2-p2*q3-M)^2)*q1^(-3)-(q1-q2)*((q1-q2)^2+q3^2)^(-1.5),"
          "(p3^2)/q1-(2*p3*(p3*q2-p2*q3-M))*(q1^(-2))+(q1-q2)*((q1-q2)^2+q3^2)^(-1.5)-q2*(q2^2+q3^2)^(-1.5),"
          "-((p2*p3)/q1)+(2*p2*(p3*q2-p2*q3-M))*q1^(-2)-q3*((q1-q2)^2+q3^2)^(-1.5)-q3*(q2^2+q3^2)^(-1.5);";
}

// whittaker 3 body equation with M=0
//std::string whittaker3BodyEquation2D(){
//   return "var:q1,q2,q3,p1,p2,p3;par:M;"
//          "fun:2*p1+p2,"
//          "p1+2*p2+(p3*q3)/q1-(2*q3*(p3*q2-p2*q3))*q1^(-2),"
//          "2*p3-(p3*q2)/q1-(p3*q2-p2*q3)/q1+(2*q2*(p3*q2-p2*q3))*q1^(-2),"
//          "-q1^(-2)-(p3*(p3*q2-p2*q3))*q1^(-2)+(2*(p3*q2-p2*q3)^2)*q1^(-3)-(q1-q2)*((q1-q2)^2+q3^2)^(-1.5),"
//          "(p3^2)/q1-(2*p3*(p3*q2-p2*q3))*(q1^(-2))+(q1-q2)*((q1-q2)^2+q3^2)^(-1.5)-q2*(q2^2+q3^2)^(-1.5),"
//          "-((p2*p3)/q1)+(2*p2*(p3*q2-p2*q3))*q1^(-2)-q3*((q1-q2)^2+q3^2)^(-1.5)-q3*(q2^2+q3^2)^(-1.5);";
//}
