
#ifndef _NBODY_H_
#define _NBODY_H_

#include <string>

///      Generates 3D N-body equation
std::string nBodyEquation3D(int numberOfBodies);

///      Generates 2D N-body equation
std::string nBodyEquation2D(int numberOfBodies);


/**
 *  Generates 2D N-body equation reduced by center of mass condition
 *
 *  Variables names and equations order :
 *  \code
 *  (q11,q12,p11,p12,q21,q22,...,q(N-1)1,q(N-1)2,p(N-1)1,p(N-1)2)
 *  \endcode
 *  where
 *  - q - positions,
 *  - p - momenta
 *
 *  e.g. q21 denotes fist coordinate of the position of the second body
 */
std::string reducedNBodyEquation2D(int numberOfBodies);

///      Generates 2D first integrals of the reduced N-body equation
std::string reducedFirstIntegrals2D(int numberOfBodies);

std::string reduced3BodyEquation2D();

///  Generates 3 body equation after whittaker reductions
std::string whittaker3BodyEquation2D();

#endif //_NBODY_H_
